@@ -6,13 +6,13 @@ import datetime
 import math
 import re
 import pymongo
-from pandas import DataFrame
+import pandas as pd
 from util_hj3415 import utils
 from .setting import load as load_setting
 from collections import OrderedDict
-import logging
 from typing import Tuple
 
+import logging
 logger = logging.getLogger(__name__)
 formatter = logging.Formatter('%(levelname)s: [%(name)s] %(message)s')
 ch = logging.StreamHandler()
@@ -441,8 +441,11 @@ class C101(Corps):
         저장된 데이터에서 가장 최근 날짜의 딕셔너리를 반환한다.
 
         """
-        d = self.client[self.db][self.col].find({'date': {'$exists': True}}).sort('date', pymongo.DESCENDING).next()
-        del d['_id']
+        try:
+            d = self.client[self.db][self.col].find({'date': {'$exists': True}}).sort('date', pymongo.DESCENDING).next()
+            del d['_id']
+        except StopIteration:
+            d = {}
         return d
 
 
@@ -530,29 +533,37 @@ class C106(Corps):
     # ======================부모 클래스에서 재구성한 함수들================
 
     def chg_page(self, page: str):
+        """
+
+        Examples:
+            c106q or c106y
+        """
         # 실제로 c106에서는 페이지가 바뀌는 것이 아니라 도큐먼트가 바뀌는 것임
         col = page[:4]
         period = page[4:5]
         if col == 'c106' and period in self.PERIOD:
-            self.page = period
+            self.page = page
         else:
             raise ValueError(f'Invalid page : {page}(c106q or c106y)')
 
     def get_all(self) -> dict:
         c106_dict = self.client[self.db][self.col].find_one({'title': self.page})
-        del c106_dict['_id']
-        del c106_dict['title']
-        del c106_dict['stamp']
-        return c106_dict
+        if c106_dict is None:
+            return {}
+        else:
+            del c106_dict['_id']
+            del c106_dict['title']
+            del c106_dict['stamp']
+            return c106_dict
 
     # ========================특정 페이지 관련 함수들=======================
 
     def find(self, title: str) -> dict:
         page_dict = self.get_all()
-        if page_dict is None:
-            return {}
-        else:
+        try:
             return page_dict[title]
+        except KeyError:
+            return {}
 
     def get_stamp(self) -> datetime.datetime:
         return self.client[self.db][self.col].find_one({'title': self.page})['stamp']
@@ -574,7 +585,7 @@ class C106(Corps):
         c106_dict['title'] = self.page
         c106_dict['stamp'] = datetime.datetime.now()
         # delete all documents in a collection
-        self.client[self.db][self.col].delete_one({'title': self.page})
+        self.client[self.db][self.col].delete_many({'title': self.page})
         result = self.client[self.db][self.col].insert_one(c106_dict)
         return result.acknowledged
 
@@ -642,9 +653,9 @@ class C1034(Corps):
             if item_dict['항목'] in not_duplicated_titles:
                 # 중복된 타이틀을 뺀 나머지 항목들은 타이틀을 키로 하고 년/월을 값으로 하는 새로운 딕셔너리로 만든다.
                 temp_dict = {}
-                # 전년/분기대비 타이틀 제거를 위해 새로운 임시 딕셔너리를 만든다.
+                # 전년/분기대비, 항목타이틀, Unnamed 제거를 위해 새로운 임시 딕셔너리를 만든다.
                 for k, v in item_dict.items():
-                    if k.startswith('전') or k.startswith('항'):
+                    if k.startswith('전') or k.startswith('항') or k.startswith('Unnamed'):
                         pass
                     else:
                         temp_dict[k] = v
@@ -701,6 +712,14 @@ class C1034(Corps):
 
     # ========================특정 페이지 관련 함수들=======================
 
+    @ staticmethod
+    def _nan_to_zero(target: Tuple[str, float]) -> Tuple[str, float]:
+        """
+
+        실수형 입력값을 받아 nan인 경우 0으로 변환하여 반환하는 유틸함수.
+        """
+        return target[0], 0 if math.isnan(target[1]) else target[1]
+
     def get_stamp(self) -> datetime.datetime:
         return self.client[self.db][self.col].find({'stamp': {'$exists': True}}).next()['stamp']
 
@@ -746,13 +765,14 @@ class C1034(Corps):
         except KeyError:
             return {}
 
-    def sum_recent_4q(self, title: str) -> Tuple[str, float]:
+    def sum_recent_4q(self, title: str, nan_to_zero: bool = False) -> Tuple[str, float]:
         """최근 4분기 합
 
         분기 페이지 한정 해당 타이틀의 최근 4분기의 합을 튜플 형식으로 반환한다.
 
         Args:
             title (str): 찾고자 하는 타이틀
+            nan_to_zero (bool): 반환값이 nan 인 경우 0으로 바꿀것인지 결정 여부
 
         Returns:
             tuple: (계산된 4분기 중 최근분기, 총합)
@@ -770,7 +790,11 @@ class C1034(Corps):
             logger.info(f'{title} : {od_q}')
             if len(od_q) < 4:
                 # od_q의 값이 4개 이하이면 그냥 최근 연도의 값으로 반환한다.
-                return C1034(code=self.db, page=self.col[:-1]+'y').latest_value(title=title)
+                t = C1034(code=self.db, page=self.col[:-1]+'y').latest_value(title=title)
+                if nan_to_zero:
+                    return self._nan_to_zero(t)
+                else:
+                    return t
             else:
                 q_sum = 0
                 last_date = list(od_q.items())[-1][0]
@@ -779,18 +803,22 @@ class C1034(Corps):
                     d, v = od_q.popitem(last=True)
                     logger.debug(f'd:{d} v:{v}')
                     q_sum += 0 if math.isnan(v) else v
-                return str(last_date), round(q_sum, 2)
-
+                t = str(last_date), round(q_sum, 2)
+                if nan_to_zero:
+                    return self._nan_to_zero(t)
+                else:
+                    return t
         else:
             raise TypeError(f'Not support year data..{self.col}')
 
-    def latest_value(self, title: str) -> Tuple[str, float]:
+    def latest_value(self, title: str, nan_to_zero: bool = False) -> Tuple[str, float]:
         """가장 최근 년/분기 값
 
         해당 타이틀의 가장 최근의 년/분기 값을 튜플 형식으로 반환한다.
 
         Args:
             title (str): 찾고자 하는 타이틀
+            nan_to_zero (bool): 반환값이 nan 인 경우 0으로 바꿀것인지 결정 여부
 
         Returns:
             tuple: ex - ('2020/09', 39617.5) or ('', 0)
@@ -807,16 +835,30 @@ class C1034(Corps):
             last_one = od.popitem(last=True)
         except KeyError:
             # when dictionary is empty
-            return '', 0
+            if nan_to_zero:
+                return '', 0
+            else:
+                return '', float('nan')
         logger.info(f'last_one : {last_one}')
         if isinstance(last_one[1], str):
             # last_one : ('Unnamed: 1', '데이터가 없습니다.') 인 경우
-            return '', 0
-        elif math.isnan(last_one[1]):
-            last_one = od.popitem(last=True)
-            if last_one is None:
+            if nan_to_zero:
                 return '', 0
-        return tuple(last_one)
+            else:
+                return '', float('nan')
+        elif math.isnan(last_one[1]):
+            try:
+                last_one = od.popitem(last=True)
+            except KeyError:
+                # when dictionary is empty
+                if nan_to_zero:
+                    return '', 0
+                else:
+                    return '', float('nan')
+        if nan_to_zero:
+            return self._nan_to_zero(tuple(last_one))
+        else:
+            return tuple(last_one)
 
 
 class C104(C1034):
@@ -928,7 +970,83 @@ class C103(C1034):
         return result.acknowledged
 
 
+class DartByCode(Corps):
+    """각 Code 별로 분류된 Dart 를 관리하는 클래스
+    """
+    def __init__(self, code: str):
+        super().__init__(code=code, page='dart')
+
+    def save(self, dart_dict: dict) -> bool:
+        """
+
+        Args:
+            dart_dict (dict) : dart 에서 전달되는 딕셔너리 구조
+
+        Note:
+            dart_dict 구조\n
+            {'rcept_no': '20210514000624',\n
+            'rcept_dt': '20210514',\n
+            'report_nm': '임원ㆍ주요주주특정증권등소유상황보고서',\n
+            'point': 2,\n
+            'text': '등기임원이 1.0억 이상 구매하지 않음.',\n
+            'is_noti': True}\n
+        """
+        self.client[self.db][self.col].delete_many({'rcept_no': {"$eq": dart_dict['rcept_no']}})
+        result = self.client[self.db][self.col].insert_one(dart_dict)
+        return result.acknowledged
+
+    def get_data(self) -> pd.DataFrame:
+        try:
+            df = pd.DataFrame(list(self.client[self.db][self.col].find())).drop(columns=['_id'])
+        except KeyError:
+            df = pd.DataFrame()
+        return df
+
+    def cleaning_data(self, days_ago: int = 180):
+        """
+
+        days_ago 인자를 기준으로 이전 날짜의 데이터를 검색하여 삭제한다.
+        본 함수를 주기적으로 실행해 준다.
+        """
+        border_date_str = (datetime.datetime.today() - datetime.timedelta(days=days_ago)).strftime('%Y%m%d')
+        logger.info(f'boder_date : {border_date_str}')
+        try:
+            self.client[self.db][self.col].delete_many({'rcept_dt': {'$lt': border_date_str}})
+            logger.info(f'Delete dart data before {days_ago} days ago..')
+        except:
+            logger.error(f'Error occurred while delete dart data..')
+
+
+class DartByDate(Base):
+    """각 날짜별로 만들어진 공시 데이터프레임을 관리하는 클래스
+    """
+    def __init__(self, date: str):
+        if utils.isYmd(date):
+            super().__init__(db='dart', col=date)
+        else:
+            raise ValueError(f"Invalid date : {date}(%Y%m%d)")
+
+    def save(self, df: pd.DataFrame) -> bool:
+        self.client[self.db][self.col].drop()
+        result = self.client[self.db][self.col].insert_many(df.to_dict('records'))
+        return result.acknowledged
+
+    def get_data(self, title: str = '') -> pd.DataFrame:
+        try:
+            df = pd.DataFrame(list(self.client[self.db][self.col].find())).drop(columns=['_id'])
+        except KeyError:
+            df = pd.DataFrame()
+        if title != '':
+            df = df[df['report_nm'].str.contains(title)]
+        return df
+
+
 class Noti(Base):
+    """텔레그램 노티한 공시자료 저장 클래스
+
+    dart를 분석하여 유의미한 공시들은 텔레그램으로 노티하며 본 클래스를 통해 데이터베이스에 저장한다.
+
+    """
     def __init__(self):
         super().__init__(db='noti', col='noti')
 
@@ -951,22 +1069,23 @@ class Noti(Base):
         result = self.client[self.db][self.col].insert_one(noti_dict)
         return result.acknowledged
 
-    def get_data(self) -> DataFrame:
+    def get_data(self) -> pd.DataFrame:
         try:
-            df = DataFrame(list(self.client[self.db][self.col].find())).drop(columns=['_id'])
+            df = pd.DataFrame(list(self.client[self.db][self.col].find())).drop(columns=['_id'])
         except KeyError:
-            df = DataFrame()
+            df = pd.DataFrame()
         return df
 
     def cleaning_data(self, days_ago: int = 15):
-        """
+        """오래된 노티 자료 삭제 함수
 
         days_ago 인자를 기준으로 이전 날짜의 데이터를 검색하여 삭제한다.
         본 함수를 주기적으로 실행해 준다.
         """
         border_date_str = (datetime.datetime.today() - datetime.timedelta(days=days_ago)).strftime('%Y%m%d')
+        logger.info(f'boder_date : {border_date_str}')
         try:
             self.client[self.db][self.col].delete_many({'rcept_dt': {'$lt': border_date_str}})
             logger.info(f'Delete noti data before {days_ago} days ago..')
         except:
-            logger.error(f'Error occurred while delete noti data..')
\ No newline at end of file
+            logger.error(f'Error occurred while delete noti data..')
