@@ -1,5 +1,5 @@
 # 파일명을 sqlite3.py로 하면 sqlite3 모듈과 이름이 중복되어 에러가 발생한다.
-from sqlalchemy import text, Column, String, Integer, create_engine, Float, TIMESTAMP, MetaData
+from sqlalchemy import text, Column, String, Integer, create_engine, Float, TIMESTAMP, MetaData, Boolean
 from sqlalchemy.sql.expression import func
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.exc import IntegrityError
@@ -60,6 +60,18 @@ class StampStructure(BaseCorps):
     sqltime = Column(TIMESTAMP(timezone=True), server_default=func.now(), onupdate=func.current_timestamp())
 
 
+class DartStructure(BaseCorps):
+    """DartByCode 의 테이블 구조 클래스
+    """
+    __tablename__ = 'dart'
+    rcept_no = Column('rcept_no', String, nullable=False, primary_key=True)
+    rcept_dt = Column('rcept_dt', String)
+    report_nm = Column('report_nm', String)
+    point = Column("point", Integer)
+    text = Column('text', String)
+    is_noti = Column('is_noti', Boolean)
+
+
 # ===========================================================
 
 
@@ -407,6 +419,37 @@ class C108(Corps):
         return True
 
 
+class DartByCode(Corps):
+    def __init__(self, code: str):
+        self.table = DartStructure.__tablename__
+        super().__init__(code=code, table='c101')
+        self.session = self._get_session(BaseCorps)
+
+    def save(self, dart_dict: dict) -> bool:
+        """
+        dart_dict 구조\n
+            {'rcept_no': '20210514000624',\n
+            'rcept_dt': '20210514',\n
+            'report_nm': '임원ㆍ주요주주특정증권등소유상황보고서',\n
+            'point': 2,\n
+            'text': '등기임원이 1.0억 이상 구매하지 않음.',\n
+            'is_noti': True}\n
+        """
+        self.session.add(DartStructure(rcept_no=dart_dict['rcept_no'],
+                                       rcept_dt=dart_dict['rcept_dt'],
+                                       report_nm=dart_dict['report_nm'],
+                                       point=dart_dict['point'],
+                                       text=dart_dict['text'],
+                                       is_noti=dart_dict['is_noti']))
+        try:
+            self.session.commit()
+            logger.info(f'Add {self.table} : {dart_dict}')
+        except IntegrityError:
+            # 같은 내용 중복저상시 경고 로그 표시하고 저장하지 않음.
+            logger.warning(f"rcept_no : {dart_dict['rcept_no']}  exists already in db.")
+        return True
+
+
 # ===========================================================
 
 
@@ -491,4 +534,35 @@ class Noti(Base):
         return df
 
     def cleaning_data(self, days_ago=15):
-        logger.warning("Deleting sqlite noti data is not implement yet..")
\ No newline at end of file
+        logger.warning("Deleting sqlite noti data is not implement yet..")
+
+
+class DartByDate(Base):
+    def __init__(self, date: str):
+        super().__init__(folder_name='dart', db_name='dart')
+        if utils.isYmd(date):
+            self.date = date
+            self.tablename = 't' + date
+        else:
+            raise ValueError(f'Invalid date : {date}(%Y%m%d)')
+
+    def save(self, df: pd.DataFrame) -> bool:
+        # 전체 공시 마감후 저녁 8시경 한번 실행함.
+        if df.empty:
+            print('Dataframe is empty..So we will skip saving db..')
+            return False
+
+        # 테이블을 저장한다.
+        print(f"Save dataframe on {self.tablename} table ...")
+        df.to_sql(self.tablename, con=self.engine, index=False, if_exists='replace')
+        return True
+
+    def get_data(self, title: str = '') -> pd.DataFrame:
+        """
+        analysis를 제작하는 동안 dart를 매번 받아오지 않도록 하는 개발전용 함수
+        """
+        df = pd.read_sql_query(text(f"SELECT * FROM t{self.date}"), con=self.engine)
+        if title != '':
+            df = df[df['report_nm'].str.contains(title)]
+        return df
+        
\ No newline at end of file
